package server;
import java.util.Enumeration;
import java.util.NoSuchElementException;

public class ByteTokenizer implements Enumeration<Object> {
 
private int currentPosition;
private int newPosition;
private int maxPosition;
 
private byte[] bytes;
private byte[] delimiters;
public ByteTokenizer(byte[] bytes, byte[] delimiters) {
this.bytes = bytes;
this.delimiters = delimiters;
currentPosition = 0;
newPosition = -1;
maxPosition = bytes.length;
}
public ByteTokenizer(byte[] bytes, byte delimiter) {
this(bytes, new byte[] { delimiter });
}
public boolean hasMoreTokens() {
newPosition = skipDelimiters(currentPosition);
return (newPosition < maxPosition);
}
public boolean hasMoreElements() {
return hasMoreTokens();
}
public Object nexToken() {
currentPosition = (newPosition >= 0) ? newPosition
: skipDelimiters(currentPosition);

newPosition = -1;
 
if (currentPosition >= maxPosition) {
throw new NoSuchElementException(
"current token position is out of bound.");
}
final int startPosition = currentPosition;
currentPosition = scanToken(currentPosition);
final int length = currentPosition - startPosition;
final byte[] token = new byte[length];
System.arraycopy(bytes, startPosition, token, 0, length);
return token;
}
public Object nextElement() {
return nexToken();
}
public int countTokens() {
int tokenNums = 0;
int position = currentPosition;
while (position < maxPosition) {
position = skipDelimiters(position);
if (position >= maxPosition) {
break;
}
position = scanToken(position);
tokenNums++;
}
return tokenNums;
}
private int skipDelimiters(final int startPosition) {
if (delimiters == null) {
throw new NullPointerException("delimiters");
}
int position = startPosition;
while (position < maxPosition) {
if (isDelimiter(position)) {
position += delimiters.length;
}
break;
}
return position;
}
private int scanToken(final int startPosition) {
int position = startPosition;
while (position < maxPosition) {
if (isDelimiter(position)) {
break;
}
position++;
}
return position;
}
 
private boolean isDelimiter(final int startPosition) {
int position = startPosition;
boolean isDelimiter = true;
while ((position < maxPosition) && isDelimiter) {
int nDelimiter = 0;
while (nDelimiter < delimiters.length) {
if (bytes[startPosition + nDelimiter] != delimiters[nDelimiter]) {
isDelimiter = false;
break;
}
nDelimiter++;
}
position++;
}
return isDelimiter;
}
}